% Experiment result of Figure 10 that is "Impact of the balancing parameter 
% on reconstructed errors when PBDW,contourlets and TIDCT are used as tight
% frames for T2-weighted brain image".
%%
clc;
clear all;
close all;
clear classes;
%%
addpath('./Data')
addpath('./Operators')
addpath('./Solvers')
addpath('./Others')
addpath('./PBDW_Toolbox')
addpath('./ContourletSD_Toolbox')
addpath('./Contourlet_Toolbox')
%%
load im_ori_slice_7.mat
load mask_center_gradual_0.4.mat
%%
[row,column]=size(im_ori);
Fu = Fu_downsample(mask,row,column);
y = Fu*im_ori;
%% balancing parameters
gamma_test = 0:0.05:1;
%% set SIDWT operator and 
filterType = 'Daubechies'; filterSize = 4;
wavescale = 4;   complex_yes=1;
params.psi = Rice_Wavelet(row,column,wavescale,filterType, filterSize,complex_yes);

params.lambda = 5e-2;     % regularization parameter
%% C-SALSA-B solver
h = waitbar(0,'Operating Experiment of SIDWT . . .');
for m=1:length(gamma_test)
    waitbar(m/length(gamma_test));
    params.gamma = gamma_test(m);   % balancing parameter which is in the range [0,1]. And gamma = 0
                                    % is of analysis model whereas gamma = 1 is of synthesis model.   
    im_rec = Solver_C_SALSA_B(y,mask,params);
    
    rlne_test_SIDWT(m) = RLNE(im_ori,im_rec);       % relative L2 norm error
end
close(h);
%% set PBDW operator and C-SALSA-B algorrithm's paramters
paramsGeo.patchSize=8;
paramsGeo.S=(paramsGeo.patchSize).^2/4;
paramsGeo.Scales=3;
paramsGeo.overlapFactor=64;
paramsGeo.row=size(im_ori,1);
paramsGeo.column=size(im_ori,2);

[paramsGeo.Lut,paramsGeo.Theta]= compute_geometric_lut_limitedAngle(paramsGeo.patchSize);

im_ref = abs(im_rec);
patches_ref=imToPatch(im_ref,[paramsGeo.patchSize paramsGeo.overlapFactor]);

tic
Geo = Est_Geo(abs(patches_ref),paramsGeo.Lut,paramsGeo.S,paramsGeo.Scales);
Est_Geo_time = toc

params.psi = PBDW_Fast(Geo,paramsGeo);

params.lambda = 5e-3;     % regularization parameter
%% C-SALSA-B solver
h = waitbar(0,'Operating Experiment of PBDW . . .');
for m=1:length(gamma_test)
    waitbar(m/length(gamma_test));
    params.gamma = gamma_test(m);   % balancing parameter which is in the range [0,1]. And gamma = 0
                                    % is of analysis model whereas gamma = 1 is of synthesis model.   
    im_rec = Solver_C_SALSA_B(y,mask,params);
    
    rlne_test_PBDW(m) = RLNE(im_ori,im_rec);       % relative L2 norm error
end
close(h);
%% set TIDCT operator and C-SALSA-B algorrithm's paramters
block_size = 8; overlap_factor = 4;
params.psi = TIDCT(block_size,overlap_factor);

params.lambda = 5e-2;     % regularization parameter
%% C-SALSA-B solver
h = waitbar(0,'Operating Experiment of TIDCT . . .');
for m=1:length(gamma_test)
    waitbar(m/length(gamma_test));
    params.gamma = gamma_test(m);   % balancing parameter which is in the range [0,1]. And gamma = 0
                                    % is of analysis model whereas gamma = 1 is of synthesis model.   
    im_rec = Solver_C_SALSA_B(y,mask,params);
    
    rlne_test_TIDCT(m) = RLNE(im_ori,im_rec);       % relative L2 norm error
end
close(h);
%% set Contourlet operator and C-SALSA-B algorrithm's paramters
SFLCT.nlevs = [5,4,4,3];
SFLCT.Pyr_mode = 1;
SFLCT.smooth_func = @rcos;
SFLCT.dfilt = 'pkva';
params.psi = ContourletSD(SFLCT.nlevs, SFLCT.Pyr_mode, SFLCT.smooth_func, SFLCT.dfilt);

params.lambda = 5e-2;     % regularization parameter
%% C-SALSA-B solver
h = waitbar(0,'Operating Experiment of Contourlet . . .');
for m=1:length(gamma_test)
    waitbar(m/length(gamma_test));
    params.gamma = gamma_test(m);   % balancing parameter which is in the range [0,1]. And gamma = 0
                                    % is of analysis model whereas gamma = 1 is of synthesis model.   
    im_rec = Solver_C_SALSA_B_Contourlet(y,mask,params);
    
    rlne_test_contourlet(m) = RLNE(im_ori,im_rec);       % relative L2 norm error
end
close(h);

%% Resulets
figure,plot(gamma_test,[rlne_test_SIDWT; rlne_test_PBDW; rlne_test_TIDCT; rlne_test_contourlet],'LineWidth',2);
legend('SIDWT','PBDW','TIDCT','Contourlet','Location','NorthWest');
xlabel('\gamma','FontSize',20);
ylabel('RLNE','FontSize',20);
grid on;set(gcf,'color','w');


%% save results
save('RLNE_VS_gamma_SIDWT_PBDW_TIDCT_Contourlet.mat','rlne_test_SIDWT','rlne_test_PBDW',...
    'rlne_test_TIDCT','rlne_test_contourlet','gamma_test');