% Experiment result of Figure 4 that is "Reconstructed T2 weighted brain 
% images using analysis, balanced and synthesis models".
%%
clc;
clear all;
close all;
clear classes;
%%
addpath('./Data')
addpath('./Operators')
addpath('./Solvers')
addpath('./Others')
%%
load im_ori_slice_7.mat
load mask_center_gradual_0.4.mat
%%
[row,column]=size(im_ori);
Fu = Fu_downsample(mask,row,column);
y = Fu*im_ori;
%% set SIDWT operator and C-SALSA-B algorrithm's paramters
filterType = 'Daubechies'; filterSize = 4;
wavescale = 4;   complex_yes=1;
params.psi = Rice_Wavelet(row,column,wavescale,filterType, filterSize,complex_yes);

%% analysis model solver
params.lambda = 5e-2;     % regularization parameter
params.gamma = 0;         % balancing parameter which is in the range [0,1]. And gamma = 0
                          % is of analysis model whereas gamma = 1 is of synthesis model.

fprintf('Solving the analysis model . . .\n');
tic
im_rec_analysis = Solver_C_SALSA_B(y,mask,params);
run_time_analysis = toc;
fprintf('Done for solving analysis model.\n\n');
%% balance model solver
params.lambda = 5e-2;     % regularization parameter
params.gamma = 1/2;       % balancing parameter which is in the range [0,1]. And gamma = 0
                          % is of analysis model whereas gamma = 1 is of synthesis model.
                          
fprintf('Solving the balance model . . .\n');
tic
im_rec_balance = Solver_C_SALSA_B(y,mask,params);
run_time_balance = toc;
fprintf('Done for balance model solver.\n\n');
%% synthesis model solver
params.lambda = 3e-1;     % regularization parameter
params.gamma = 1;         % balancing parameter which is in the range [0,1]. And gamma = 0
                          % is of analysis model whereas gamma = 1 is of synthesis model.
                          
fprintf('Solving the synthesis model . . .\n');
tic
im_rec_synthesis = Solver_C_SALSA_B(y,mask,params);
run_time_synthesis = toc;
fprintf('Done for synthesis model solver.\n\n');
%% Results
% relative L2 norm error
rlne_err_analysis = RLNE(im_ori,im_rec_analysis);
rlne_err_balance = RLNE(im_ori,im_rec_balance);
rlne_err_synthesis = RLNE(im_ori,im_rec_synthesis);

figure,
subplot(2,4,1),imshow(abs(im_ori),[]),title('Original image');
subplot(2,4,2),imshow(abs(im_rec_analysis),[]),title(sprintf('Recon of analysis model (RLNE:%.3f. CPU time: %2.1f s)',rlne_err_analysis,run_time_analysis));
subplot(2,4,3),imshow(abs(im_rec_balance),[]),title(sprintf('Recon of balance model (RLNE:%.3f. CPU time: %2.1f s)',rlne_err_balance,run_time_balance));
subplot(2,4,4),imshow(abs(im_rec_synthesis),[]),title(sprintf('Recon of synthesis model (RLNE:%.3f. CPU time: %2.1f s)',rlne_err_synthesis,run_time_synthesis));
subplot(2,4,5),imshow(abs(mask),[]),title('40% sampling pattern');
subplot(2,4,6),imshow(6*abs(im_ori-im_rec_analysis),[]),title('Difference image of analysis model');
subplot(2,4,7),imshow(6*abs(im_ori-im_rec_balance),[]),title('Difference image of balance model');
subplot(2,4,8),imshow(6*abs(im_ori-im_rec_synthesis),[]),title('Difference image of synthesis model');