% Experiment result of Figure 6 that is "Reconstruction error RLNEs in the
% iterations using different algorithms".
%%
clc;
clear all;
close all;
clear classes;
%%
addpath('./Data')
addpath('./Operators')
addpath('./Solvers')
addpath('./Others')
%%
load im_ori_slice_7.mat
load mask_center_gradual_0.4.mat
%%
[row,column]=size(im_ori);
Fu = Fu_downsample(mask,row,column);
y = Fu*im_ori;
%% set SIDWT operator and C-SALSA-B algorrithm's paramters
filterType = 'Daubechies'; filterSize = 4;
wavescale = 4;   complex_yes=1;
params.psi = Rice_Wavelet(row,column,wavescale,filterType, filterSize,complex_yes);

params.im_ori = im_ori;   % used to calculate the relative L2
%% C-SALSA-B Solver
params.lambda = 5e-2;        % regularization parameter
params.tol_error = 1e-20;    % to fix the number of iters to be iter_mat
params.iter_max = 200;

fprintf('C-SALSA-B is operating . . .\n');
[im_rec_C_SALSA_B,rlne_iter_C_SALSA_B] = Solver_C_SALSA_B_rlne_iter(y,mask,params);
fprintf('Done for C-SALSA-B solver.\n\n');
%% ADMM-B Solver
params.lambda = 1e-2;        % regularization parameter
params.tol_error = 1e-20;    % to fix the number of iters to be iter_mat
params.iter_max = 200;

fprintf('ADMM-B is operating . . .\n');
[im_rec_ADMM_B,rlne_iter_ADMM_B] = Solver_ADMM_B_rlne_iter(y,mask,params);
fprintf('Done for ADMM-B solver.\n\n');
%% APG Solver
params.lambda = 5e-3;        % regularization parameter
params.tol_error = 1e-20;    % to fix the number of iters to be iter_mat
params.iter_max = 200;

fprintf('APG is operating . . .\n');
[im_rec_APG,rlne_iter_APG] = Solver_APG_rlne_iter(y,mask,params);
fprintf('Done for APG solver.\n\n');
%% Resulets
figure,plot(1:params.iter_max,[rlne_iter_APG, rlne_iter_ADMM_B, rlne_iter_C_SALSA_B],'LineWidth',2);
legend('APG','ADMM-B','C-SALSA-B');
xlabel('# iter','FontSize',20);
ylabel('RLNE','FontSize',20);
grid on;set(gcf,'color','w');