% Experiment result of Figure 8 that is "Comparisons of three models for
% dfferent percentages of acquired k-space data.".
%%
clc;
clear all;
close all;
clear classes;
%%
addpath('./Data')
addpath('./Operators')
addpath('./Solvers')
addpath('./Others')
%%
load im_ori_slice_7.mat
%%
[row,column]=size(im_ori);
%% set SIDWT operator and C-SALSA-B algorrithm's paramters
filterType = 'Daubechies'; filterSize = 4;
wavescale = 4;   complex_yes=1;
params.psi = Rice_Wavelet(row,column,wavescale,filterType, filterSize,complex_yes);

%% C-SALSA solver
h = waitbar(0,'Experimenting on analysis model . . .');
rate = [0.15:0.05:0.5 0.6:0.1:1];
for m=1:length(rate)
    waitbar(m/length(rate));
    load(sprintf('mask_center_gradual_%s.mat',num2str(rate(m))));
    
    Fu = Fu_downsample(mask,row,column);
    y = Fu*im_ori;
    %%%%%%%% analysis model
    params.lambda = 5e-2;      % regularization parameter
    params.gamma = 0;        % balancing parameter which is in the range [0,1]. And gamma = 0
                               % is of analysis model whereas gamma = 1 is of synthesis model.
    im_rec = Solver_C_SALSA_B(y,mask,params);
    rlne_test_analysis(m) = RLNE(im_ori,im_rec);       % relative L2 norm error
    %%%%%%%% balance model
    params.lambda = 5e-2;      % regularization parameter
    params.gamma = 1/2;        % balancing parameter which is in the range [0,1]. And gamma = 0
                               % is of analysis model whereas gamma = 1 is of synthesis model.
    im_rec = Solver_C_SALSA_B(y,mask,params);
    rlne_test_balance(m) = RLNE(im_ori,im_rec);       % relative L2 norm error
    %%%%%%%% analysis model
    params.lambda = 5e-2;      % regularization parameter
    params.gamma = 1;        % balancing parameter which is in the range [0,1]. And gamma = 0
                               % is of analysis model whereas gamma = 1 is of synthesis model.
    im_rec = Solver_C_SALSA_B(y,mask,params);
    rlne_test_synthesis(m) = RLNE(im_ori,im_rec);       % relative L2 norm error
end
close(h);

%% Resulets
figure,
h = plot(rate,[rlne_test_analysis;rlne_test_balance;rlne_test_synthesis],'LineWidth',2);
legend(h,'Analysis model','Balance synthesis model','Synthesis model');
xlabel('Percentages of data acquired','FontSize',20);
ylabel('RLNE','FontSize',20);
title('RLNE V.S. percentages of data acquired','FontSize',20);
grid on;set(gcf,'color','w');