% Experiment result of Figure 9 that is "Comparisons on PBDW-based 
% reconstructed images for three models.".
%%
clc;
clear all;
close all;
clear classes;
%%
addpath('./Data')
addpath('./Operators')
addpath('./Solvers')
addpath('./PBDW_Toolbox')
addpath('./Others')
%%
load im_ori_slice_7.mat
load mask_center_gradual_0.4.mat
%%
[row,column]=size(im_ori);
Fu = Fu_downsample(mask,row,column);
y = Fu*im_ori;
%% set SIDWT operator and C-SALSA-B algorrithm's paramters
filterType = 'Daubechies'; filterSize = 4;
wavescale = 4;   complex_yes=1;
params_SIDWT.psi = Rice_Wavelet(row,column,wavescale,filterType, filterSize,complex_yes);

params_SIDWT.lambda = 5e-2;     % regularization parameter
params_SIDWT.gamma = 1/2;       % balancing parameter which is in the range [0,1]. And gamma = 0
                          % is of analysis model whereas gamma = 1 is of synthesis model.

fprintf('Computing SIDWT based reconstruction as the reference image . . .\n');
tic
im_rec_SIDWT = Solver_C_SALSA_B(y,mask,params_SIDWT);
run_time_SIDWT = toc;
fprintf('Done for SIDWT based reconstruction.\n\n');
%% set PBDW operator and C-SALSA-B algorrithm's paramters
paramsGeo.patchSize = 8;
paramsGeo.S = (paramsGeo.patchSize).^2/4;
paramsGeo.Scales = 3;
paramsGeo.overlapFactor = 64;
paramsGeo.row = row;
paramsGeo.column = column;

[paramsGeo.Lut,paramsGeo.Theta]= compute_geometric_lut_limitedAngle(paramsGeo.patchSize);

im_ref = abs(im_rec_SIDWT);   % Using the SIDWT's reconstruction as the reference image of PBDW
patches_ref=imToPatch(im_ref,[paramsGeo.patchSize paramsGeo.overlapFactor]);

tic
Geo = Est_Geo(abs(patches_ref),paramsGeo.Lut,paramsGeo.S,paramsGeo.Scales);
Est_Geo_time = toc

params_PBDW.psi = PBDW_Fast(Geo,paramsGeo);

%% analysis model
params_PBDW.lambda = 5e-3;     % regularization parameter
params_PBDW.gamma = 0;         % balancing parameter which is in the range [0,1]. And gamma = 0
                               % is of analysis model whereas gamma = 1 is of synthesis model.
                               
fprintf('Solving the PBDW based analysis model . . .\n');
tic
im_rec_PBDW_analysis = Solver_C_SALSA_B(y,mask,params_PBDW);
run_time_PBDW_analysis = toc;
fprintf('Done for solving PBDW based analysis model\n\n');
%% balance model
params_PBDW.lambda = 5e-3;     % regularization parameter
params_PBDW.gamma = 1/2;       % balancing parameter which is in the range [0,1]. And gamma = 0
                               % is of analysis model whereas gamma = 1 is of synthesis model.
                               
fprintf('Solving the PBDW based balance model . . .\n');
tic
im_rec_PBDW_balance = Solver_C_SALSA_B(y,mask,params_PBDW);
run_time_PBDW_balance = toc;
fprintf('Done for solving PBDW based balance model\n\n');
%% synthesis model
params_PBDW.lambda = 3e-3;     % regularization parameter
params_PBDW.gamma = 1;       % balancing parameter which is in the range [0,1]. And gamma = 0
                               % is of analysis model whereas gamma = 1 is of synthesis model.
                               
fprintf('Solving the PBDW based synthesis model . . .\n');
tic
im_rec_PBDW_synthesis = Solver_C_SALSA_B(y,mask,params_PBDW);
run_time_PBDW_synthesis = toc;
fprintf('Done for solving PBDW based synthesis model\n\n');
%% Resulets
%% Results
% relative L2 norm error
rlne_err_analysis = RLNE(im_ori,im_rec_PBDW_analysis);
rlne_err_balance = RLNE(im_ori,im_rec_PBDW_balance);
rlne_err_synthesis = RLNE(im_ori,im_rec_PBDW_synthesis);

figure,
subplot(2,4,1),imshow(abs(im_ori),[]),title('Original image');
subplot(2,4,2),imshow(abs(im_rec_PBDW_analysis),[]),title(sprintf('Recon of analysis model (RLNE:%.3f. CPU time: %2.1f s)',rlne_err_analysis,run_time_PBDW_analysis));
subplot(2,4,3),imshow(abs(im_rec_PBDW_balance),[]),title(sprintf('Recon of balance model (RLNE:%.3f. CPU time: %2.1f s)',rlne_err_balance,run_time_PBDW_balance));
subplot(2,4,4),imshow(abs(im_rec_PBDW_synthesis),[]),title(sprintf('Recon of synthesis model (RLNE:%.3f. CPU time: %2.1f s)',rlne_err_synthesis,run_time_PBDW_synthesis));
subplot(2,4,5),imshow(abs(mask),[]),title('40% sampling pattern');
subplot(2,4,6),imshow(6*abs(im_ori-im_rec_PBDW_analysis),[]),title('Difference image of analysis model');
subplot(2,4,7),imshow(6*abs(im_ori-im_rec_PBDW_balance),[]),title('Difference image of balance model');
subplot(2,4,8),imshow(6*abs(im_ori-im_rec_PBDW_synthesis),[]),title('Difference image of synthesis model');